/*
    Copyright 2005 Martin Ruckert
    
    ruckertm@acm.org

    This file is part of mp32pcm.

    mp32pcm is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    mp32pcm is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with mp32pcm; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "mp32pcm.h"                                                 /*   6 */
#define WINDOWBLOCKS 16                                              /*   1 */
#define SHIFTBLOCKS (2*BLOCKS + WINDOWBLOCKS-1)
#define SHIFTSIZE (SHIFTBLOCKS*SUBBANDS)
#define CHANNELS 2
extern void dct32 (const double *y, double *v);                      /*   5 */

extern void windowing (const double *v, mp3_sample * x);

#define STREAMS 512                                                  /*  41 */
#define END_OF_OUTPUT           0x0001                               /*  59 */
#define END_OF_INPUT  0x0002
#define LAYER_I   1                                                  /*  64 */
#define LAYER_II  2
#define LAYER_III 3
#define SUBBANDS 32                                                  /*  68 */
#define SUBFREQUENCIES 18
#define BLOCKS SUBFREQUENCIES
#define HEADER_SIZE 4                                                /*  77 */
#define BITALLOCATION_ERROR 0x0008                                   /* 109 */
#define OUTPUT_EXPONENT (sizeof(int)*8-1)                            /* 137 */
#define BUFFERSIZE (2*MAX_FRAME+HEADER_SIZE+MAX_RESERVOIR)           /* 148 */
#define MAX_FRAME 1729                                               /* 170 */
static int
tag_read (int id, void *buffer, int count)                           /*  38 */
 ;

#define GROUPS 3                                                     /* 202 */
     static unsigned short int                                       /* 211 */
       bitcrc (unsigned short int crc, unsigned short int bits, int n);

     extern void dct18 (const double *z, double *t);                 /* 219 */

#define LONG_BLOCK  0                                                /* 225 */
#define START_BLOCK 1
#define SHORT_BLOCK 2
#define STOP_BLOCK  3
     extern void dct6 (const double *z, double *t);                  /* 227 */

#define FREQUENCIES 576                                              /* 235 */
#define utothreequarter(ch,i) ((uv[ch][i]>=0)? power43[uv[ch][i]]:-power43[-uv[ch][i]]) /* 239 */
#define POWER43SIZE ((1<<13)+15)                                     /* 240 */
#define BANDS 39                                                     /* 242 */
#define GRANULES 2                                                   /* 247 */
#define twotomquarter(m)  power14[m+POWER14START]                    /* 250 */
#define SUBBLOCKS 3                                                  /* 264 */
#define POWER14START (326 - 4*OUTPUT_EXPONENT)                       /* 266 */
#define POWER14SIZE  (326 + 1 + 45)                                  /* 267 */
#define STEREO    0                                                  /* 280 */
#define INTENSITY 1
#define MID_SIDE  2
#define NONE 3                                                       /* 286 */
#define HWIDTH 6                                                     /* 298 */
#define HUFFMAN_CACHE_SIZE (sizeof(int)*8)                           /* 312 */
#define MAX_RESERVOIR 512                                            /* 321 */
#define BANDGROUPS 4                                                 /* 334 */
#define REGIONS 3                                                    /* 350 */
#define INTENSITY_V2 3                                               /* 387 */
#define LAYER_III_V2 4                                               /* 396 */
#define MUTE     5                                                   /* 400 */
#define REPEAT   6
#define REPAIR   7
#define SKIP     8
#ifndef max                                                          /* 437 */
#define max(a,b) ((a) >  (b) ? (a) : (b))
#endif
#ifndef min
#define min(a,b) ((a) >  (b) ? (b) : (a))
#endif
#ifndef M_PI
#define M_PI 3.14159265358979323846264338328
#endif
#define twotom32th(m)  power132[m]                                   /* 449 */
#include <math.h>                                                    /* 138 */
     static mp3_sample mk_sample (register double x)
{
  if (fabs (x) <= ((double) ((unsigned) 1 << OUTPUT_EXPONENT))       /* 136 */
    ) {
    if (sizeof (mp3_sample) == sizeof (int))
      return (mp3_sample) x;
    else {
      const int d = (sizeof (int) - sizeof (mp3_sample)) * 8;

      return ((int) x + (1 << (d - 1))) >> d;
    }
  }
  else if (x > 0)
    return ((signed int)                                             /* 140 */
            ((((unsigned int) 1) << (sizeof (mp3_sample) * 8 - 1)) - 1));
  else
    return -((signed int)                                            /* 140 */
             ((((unsigned int) 1) << (sizeof (mp3_sample) * 8 - 1)) - 1))
      - 1;
}

void
windowing (const double *v, mp3_sample * x)
{ {                                                                  /* 412 */
    x[2 * 0] =
      mk_sample (0.0003128982862089532 * v[48] + 0.002298183305883584 * v[80] +
                 0.004952423160365843 * v[112] + 0.021978401160177793 * v[144] +
                 0.0555987738287977 * v[176] + 0.07093078561039573 * v[208] +
                 0.4044910600631663 * v[240] + 0.8096294961834989 * v[272] -
                 0.4044910600631663 * v[304] + 0.07093078561039573 * v[336] -
                 0.0555987738287977 * v[368] + 0.021978401160177793 * v[400] -
                 0.004952423160365843 * v[432] + 0.002298183305883584 * v[464] -
                 0.0003128982862089532 * v[496]);
    x[2 * 1] =
      mk_sample (-0.000011360878959224075 * v[17] +
                 0.00031919919129172973 * v[47] + 0.002476637364442382 * v[81] +
                 0.005344017796184724 * v[111] + 0.022721443724141786 * v[145] +
                 0.05680721448372963 * v[175] + 0.06769854183592361 * v[209] +
                 0.4050332784854463 * v[239] + 0.8519632570908607 * v[273] -
                 0.36697645997870415 * v[303] + 0.08104738996801669 * v[337] -
                 0.049300877952645254 * v[367] + 0.02343716941960138 * v[401] -
                 0.004128999734678011 * v[431] +
                 0.0023630300639226835 * v[465] -
                 0.00026771603849710295 * v[495]);
    x[2 * 2] =
      mk_sample (-0.000012026451399242627 * v[18] +
                 0.0003454403479776696 * v[46] + 0.002669835167364561 * v[82] +
                 0.005734305378041188 * v[110] + 0.023475308411616744 * v[146] +
                 0.058024067210062064 * v[174] + 0.06359499620678986 * v[210] +
                 0.4063954732221264 * v[238] + 0.9002396009802017 * v[274] -
                 0.3335076121914026 * v[302] + 0.09188081818496067 * v[338] -
                 0.04367349866115977 * v[366] + 0.025014672910544967 * v[402] -
                 0.003424791978534475 * v[430] +
                 0.0024293092919982022 * v[466] -
                 0.00023687325210604534 * v[494]);
    x[2 * 3] =
      mk_sample (-0.000012807636239395524 * v[19] +
                 0.00036094905567626167 * v[45] +
                 0.0028816787044306583 * v[83] + 0.006126635156083047 * v[109] +
                 0.024244520500667208 * v[147] + 0.05924313659295743 * v[173] +
                 0.05841482143352409 * v[211] + 0.40849933316132825 * v[237] +
                 0.9558206795039699 * v[275] - 0.30345366228164333 * v[301] +
                 0.10363795967934386 * v[339] - 0.03859305196361568 * v[365] +
                 0.026729167517774474 * v[403] -
                 0.0027926057862843277 * v[429] +
                 0.0025102622895860605 * v[467] -
                 0.00019947210866493324 * v[493]);
    x[2 * 4] =
      mk_sample (-0.000013732737188838566 * v[20] +
                 0.0003762073747154337 * v[44] + 0.0031172881430076437 * v[84] +
                 0.006523990077013419 * v[108] + 0.02502070156719418 * v[148] +
                 0.06045931174897625 * v[172] + 0.05185409834400458 * v[212] +
                 0.41126829825209865 * v[236] + 1.0205067915868626 * v[276] -
                 0.27630153237206495 * v[300] + 0.11661679235020145 * v[340] -
                 0.033996319386491995 * v[364] + 0.028632361049190227 * v[404] -
                 0.002238894209067779 * v[428] + 0.002609184066830402 * v[468] -
                 0.0001743401055855475 * v[492]);
    x[2 * 5] =
      mk_sample (-0.00001484041874153566 * v[21] +
                 0.000400269964178869 * v[43] + 0.003383568789794713 * v[85] +
                 0.006929113044274127 * v[107] + 0.025807131258987214 * v[149] +
                 0.06168600462095256 * v[171] + 0.043556026986667926 * v[213] +
                 0.4146527011613699 * v[235] + 1.0968105060105893 * v[277] -
                 0.2516273090155306 * v[299] + 0.13118748783541162 * v[341] -
                 0.029806748547179615 * v[363] + 0.03079344290379831 * v[405] -
                 0.0017522914386632425 * v[427] +
                 0.002715758699539749 * v[469] -
                 0.00015121274781770293 * v[491]);
    x[2 * 6] =
      mk_sample (-0.000016184880810526708 * v[22] +
                 0.00042389328289552916 * v[42] +
                 0.0036901019122711613 * v[86] + 0.007335945692136624 * v[106] +
                 0.026607575997348865 * v[150] + 0.06290054423507843 * v[170] +
                 0.03296814611963793 * v[214] + 0.4186160561314836 * v[234] +
                 1.1881965990501728 * v[278] - 0.22909258929059503 * v[298] +
                 0.14791158074082497 * v[342] - 0.025987241635379382 * v[362] +
                 0.033291839493137715 * v[406] -
                 0.0013235847089620946 * v[426] +
                 0.0028484997775783186 * v[470] -
                 0.0001384142836041004 * v[490]);
    x[2 * 7] =
      mk_sample (-0.00003568896788850478 * v[23] +
                 0.00044730397038539186 * v[41] + 0.004050641722228487 * v[87] +
                 0.007756080247117002 * v[105] + 0.02739090395588898 * v[151] +
                 0.06411636834739066 * v[169] + 0.019307464995376257 * v[215] +
                 0.4231410221445687 * v[233] + 1.2996850208044854 * v[279] -
                 0.20840984504257087 * v[297] + 0.1675395442615175 * v[343] -
                 0.022474909442567505 * v[361] + 0.03625948968498941 * v[407] -
                 0.0009368064494148356 * v[425] +
                 0.0030156756867410497 * v[471] -
                 0.00011815571358256615 * v[489]);
    x[2 * 8] =
      mk_sample (-0.000039873688562097255 * v[24] +
                 0.00047896396059038583 * v[40] + 0.004465791710495543 * v[88] +
                 0.008183675180516712 * v[104] + 0.028190308457639226 * v[152] +
                 0.06532075706446365 * v[168] + 0.0013955595012289305 * v[216] +
                 0.428201925227916 * v[232] + 1.438802225110984 * v[280] -
                 0.18934761605173117 * v[296] + 0.19123156716694048 * v[344] -
                 0.019241133583379637 * v[360] + 0.03989307403680713 * v[408] -
                 0.0005945758950919168 * v[424] +
                 0.0032097875061080226 * v[472] -
                 0.00010735382320940022 * v[488]);
    x[2 * 9] =
      mk_sample (-0.00004529373576067573 * v[25] +
                 0.0005104933669849739 * v[39] + 0.005004888045817701 * v[89] +
                 0.00862166339333471 * v[103] + 0.028987590162386057 * v[153] +
                 0.06651807580330911 * v[167] - 0.022601260985843145 * v[217] +
                 0.43378958256256167 * v[231] + 1.6174169450737852 * v[281] -
                 0.17169588389745313 * v[295] + 0.2208039089066784 * v[345] -
                 0.016254752237246636 * v[359] + 0.04443254033707173 * v[409] -
                 0.00029171019196707585 * v[423] +
                 0.003487570160304309 * v[473] - 0.000089133905786647 * v[487]);
    x[2 * 10] =
      mk_sample (-0.000052565640544037485 * v[26] +
                 0.0005421435178641913 * v[38] + 0.005650728848364608 * v[90] +
                 0.009064955437752093 * v[102] + 0.029725458062796715 * v[154] +
                 0.06769616305329242 * v[166] - 0.055771372960923764 * v[218] +
                 0.43991742826671476 * v[230] + 1.8552786357645104 * v[282] -
                 0.15529217543776508 * v[294] + 0.2592238736529565 * v[346] -
                 0.013489801838191559 * v[358] + 0.05043603451092918 * v[410] -
                 0.000015945612730729426 * v[422] +
                 0.00386352118412671 * v[474] -
                 0.00007972701865641724 * v[486]);
    x[2 * 11] =
      mk_sample (-0.00009419698351162733 * v[27] +
                 0.0005741526349443321 * v[37] + 0.006531042987258414 * v[91] +
                 0.009516773268488255 * v[101] + 0.030457268172845685 * v[155] +
                 0.06885896591478574 * v[165] - 0.1036175132063181 * v[219] +
                 0.4465647483763325 * v[229] + 2.1878680874779404 * v[283] -
                 0.1399909467378425 * v[293] + 0.31195151197178694 * v[347] -
                 0.01091676115086594 * v[357] + 0.05871659164002599 * v[411] +
                 0.00022808801187731097 * v[421] +
                 0.004364495904242906 * v[475] -
                 0.0000707858636243641 * v[485]);
    x[2 * 12] =
      mk_sample (-0.00011732001753581515 * v[28] +
                 0.000614530022057634 * v[36] + 0.007821397720968721 * v[92] +
                 0.009980281306807181 * v[100] + 0.03105094828588781 * v[156] +
                 0.06999421114046105 * v[164] - 0.1772719689146902 * v[220] +
                 0.4537644272322566 * v[228] + 2.686337095474546 * v[284] -
                 0.12566753494641256 * v[292] + 0.38974022501082417 * v[348] -
                 0.008517855199218877 * v[356] + 0.07105739463003174 * v[412] +
                 0.00044339521472142154 * v[420] +
                 0.005162121778223031 * v[476] -
                 0.00006223074634124471 * v[484]);
    x[2 * 13] =
      mk_sample (-0.00020798337982123825 * v[29] +
                 0.0006555943185742344 * v[35] + 0.009827239401714542 * v[93] +
                 0.010458658182621363 * v[99] + 0.03145756542485802 * v[157] +
                 0.07110499217425234 * v[163] - 0.3025126011809063 * v[221] +
                 0.46152299958705484 * v[227] + 3.516436002235974 * v[285] -
                 0.11220690178333473 * v[291] + 0.5178279446177007 * v[349] -
                 0.006278279899935265 * v[355] + 0.09146092554946829 * v[413] +
                 0.0006401683558200812 * v[419] +
                 0.006499498509357893 * v[477] -
                 0.000053990364168576435 * v[483]);
    x[2 * 14] =
      mk_sample (-0.0003113486059416926 * v[30] +
                 0.0006976342998365635 * v[34] + 0.013777212796247383 * v[94] +
                 0.010947490659780413 * v[98] + 0.031290613964943696 * v[158] +
                 0.07218597419618838 * v[162] - 0.5568484074977561 * v[222] +
                 0.4698604680797334 * v[226] + 5.175716801105264 * v[286] -
                 0.09950870260152139 * v[290] + 0.7718351682739714 * v[350] -
                 0.00417813888071215 * v[354] + 0.13216782244863165 * v[414] +
                 0.000812629030897881 * v[418] + 0.009106968403676138 * v[478] -
                 0.00005366440879632929 * v[482]);
    x[2 * 15] =
      mk_sample (-0.0007774364797779736 * v[31] +
                 0.0007409439994969087 * v[33] + 0.025344414974750568 * v[95] +
                 0.011442616132258705 * v[97] + 0.02876512937176878 * v[159] +
                 0.07321593830416676 * v[161] - 1.3278605597900235 * v[223] +
                 0.47881772391819916 * v[225] + 10.151758277274979 * v[287] -
                 0.08750011221721796 * v[289] + 1.5296829232042684 * v[351] -
                 0.0021999153944375154 * v[353] + 0.2540660561332939 * v[415] +
                 0.0009701015297780469 * v[417] +
                 0.017259081699064517 * v[479] -
                 0.000045831706297426926 * v[481]);
    x[2 * 16] =
      mk_sample (0.000793457 * v[32] + 0.011955261 * v[96] +
                 0.0742111205 * v[160] + 0.4884262085 * v[224] -
                 0.0761032105 * v[288] - 0.000343323 * v[352] +
                 0.0011138915 * v[416] - 0.000038147 * v[480]);
    x[2 * 17] =
      mk_sample (0.000932925813735193 * v[31] + 0.000847884313788906 * v[33] -
                 0.01974687028234753 * v[95] + 0.012481464971181395 * v[97] +
                 0.04478030658918366 * v[159] + 0.07514850316317577 * v[161] +
                 1.781105701420322 * v[223] + 0.49872390378363474 * v[225] -
                 9.746558677486934 * v[287] - 0.06523360623560533 * v[289] -
                 1.4903446617835223 * v[351] + 0.001413140189246178 * v[353] -
                 0.23291980222534767 * v[415] + 0.0012450912669596103 * v[417] -
                 0.015082261593687814 * v[479] -
                 0.000038193005147356064 * v[481]);
    x[2 * 18] =
      mk_sample (0.0005448638862594261 * v[30] + 0.0008969580984074182 * v[34] -
                 0.008250761012623637 * v[94] + 0.01301739431162627 * v[98] +
                 0.04242135595896376 * v[158] + 0.07601912888371658 * v[162] +
                 1.0103288125631422 * v[222] + 0.5097636111201113 * v[226] -
                 4.77057339081518 * v[286] - 0.05484478111165616 * v[290] -
                 0.732916495609436 * v[350] + 0.003081856481314591 * v[354] -
                 0.1111517379855069 * v[414] + 0.001356937022652688 * v[418] -
                 0.007083199914480826 * v[478] -
                 0.000030665161132494085 * v[482]);
    x[2 * 19] =
      mk_sample (0.00036397347039348086 * v[29] + 0.00096410952988962 * v[35] -
                 0.004315664502213845 * v[93] + 0.01356694671255165 * v[99] +
                 0.04232472513328717 * v[157] + 0.0768125195398193 * v[163] +
                 0.7564374879314253 * v[221] + 0.5216140070070716 * v[227] -
                 3.111335335720492 * v[285] - 0.04487350543896398 * v[291] -
                 0.4793509205301274 * v[349] + 0.0046662890328484875 * v[355] -
                 0.07050654636264594 * v[413] + 0.0014577332614290849 * v[419] -
                 0.004419657895928673 * v[477] -
                 0.00003085142003734658 * v[483]);
    x[2 * 20] =
      mk_sample (0.00031285508870580557 * v[28] +
                 0.0010268098636084333 * v[36] - 0.002229098273588622 * v[92] +
                 0.014134194585690216 * v[100] + 0.04282214983124253 * v[156] +
                 0.07752415081998365 * v[164] + 0.6317733613142048 * v[220] +
                 0.5343456712406365 * v[228] - 2.2812278251465887 * v[284] -
                 0.03526158700687418 * v[292] - 0.3518846618866347 * v[348] +
                 0.006176417632929279 * v[356] - 0.0501742623562371 * v[412] +
                 0.0015557727368957506 * v[420] -
                 0.003089451048987311 * v[476] -
                 0.000023336402429071988 * v[484]);
    x[2 * 21] =
      mk_sample (0.00028259300831589196 * v[27] + 0.001093249311821209 * v[37] -
                 0.0009105783858094341 * v[91] + 0.014707740552704334 * v[101] +
                 0.04358215350822427 * v[155] + 0.07813978596089458 * v[165] +
                 0.5588751870793057 * v[219] + 0.5480324265319146 * v[229] -
                 1.782786409461297 * v[283] - 0.025954835905633078 * v[293] -
                 0.2749004014508211 * v[347] + 0.007629148520347515 * v[357] -
                 0.03799308858720617 * v[411] + 0.0016359411047571037 * v[421] -
                 0.0022921458047390248 * v[475] -
                 0.000023595116059018058 * v[485]);
    x[2 * 22] =
      mk_sample (0.0002628247578259909 * v[26] + 0.0011719863382515497 * v[38] +
                 0.000052565640544037485 * v[90] +
                 0.015299603803195833 * v[102] + 0.044469916986641635 * v[154] +
                 0.07863470238447309 * v[166] + 0.5119296957232544 * v[218] +
                 0.5627926213267186 * v[230] - 1.4502134094076737 * v[282] -
                 0.016918061027924428 * v[294] - 0.2231643420269582 * v[346] +
                 0.009017119122058518 * v[358] - 0.029883153261981734 * v[410] +
                 0.0017141298561157408 * v[422] -
                 0.0017872076642378991 * v[474] -
                 0.000015945612730729426 * v[486]);
    x[2 * 23] =
      mk_sample (0.00024911257835448383 * v[25] +
                 0.0012478725568426936 * v[39] + 0.0008152753703752325 * v[89] +
                 0.015898217145142637 * v[103] + 0.045428989166325054 * v[153] +
                 0.07900490189542053 * v[167] + 0.479857849300644 * v[217] +
                 0.5787210366984598 * v[231] - 1.2123606659191597 * v[281] -
                 0.008086860489657418 * v[295] - 0.18590556785657805 * v[345] +
                 0.010371925607222346 * v[359] - 0.024095934971805434 * v[409] +
                 0.0017907775773322414 * v[423] -
                 0.001426733382321273 * v[473] -
                 0.000016206357794878032 * v[487]);
    x[2 * 24] =
      mk_sample (0.0002591750559647375 * v[24] + 0.001329537517365657 * v[40] +
                 0.0014354331897910278 * v[88] + 0.01652425231079951 * v[104] +
                 0.046452205652427546 * v[152] + 0.07921070867440823 * v[168] +
                 0.45712558267510284 * v[216] + 0.5959713952135566 * v[232] -
                 1.0337708953195046 * v[280] + 0.0005780596725076552 * v[296] -
                 0.15769825760950626 * v[344] + 0.011676808090635137 * v[360] -
                 0.01975713961085953 * v[408] + 0.0018497914932205967 * v[424] -
                 0.0011563212895452418 * v[472] -
                 0.000016516222584261643 * v[488]);
    x[2 * 25] =
      mk_sample (0.00024981926689973314 * v[23] +
                 0.0014263083753403834 * v[41] + 0.0019807108206935462 * v[87] +
                 0.017149461412794647 * v[105] + 0.047519203518301516 * v[151] +
                 0.07924030300458251 * v[169] + 0.4406455949067552 * v[215] +
                 0.6147052345940991 * v[233] - 0.8946565235162459 * v[279] +
                 0.009131750854567638 * v[297] - 0.13556267108171202 * v[343] +
                 0.012954932750957705 * v[361] - 0.016398853873420858 * v[407] +
                 0.001915810854369827 * v[425] -
                 0.0009457447852061088 * v[471] -
                 0.000016879624699179178 * v[489]);
    x[2 * 26] =
      mk_sample (0.00025895491093536933 * v[22] +
                 0.0015225548518689649 * v[42] + 0.0024762528223246346 * v[86] +
                 0.01779485894536847 * v[106] + 0.048618709485169304 * v[150] +
                 0.07906038704205563 * v[170] + 0.4286020886788143 * v[214] +
                 0.6351043138911755 * v[234] - 0.7831755560842464 * v[278] +
                 0.017621841232599356 * v[298] - 0.11767864129376937 * v[342] +
                 0.014222045671239966 * v[362] - 0.013724589066020854 * v[406] +
                 0.001972400564902248 * v[426] -
                 0.0007930485529389487 * v[470] -
                 8.650999027262802e-6 * v[490]);
    x[2 * 27] =
      mk_sample (0.00025228322833315475 * v[21] +
                 0.0016277631219501435 * v[43] + 0.002923521644216535 * v[85] +
                 0.018456879385188026 * v[107] + 0.04972955559714183 * v[149] +
                 0.07863075419622369 * v[171] + 0.41981480246462693 * v[213] +
                 0.6574029179227462 * v[235] - 0.6918062372106849 * v[277] +
                 0.02610647789863614 * v[299] - 0.10291688110521781 * v[341] +
                 0.01546819920113868 * v[363] - 0.01156052669846527 * v[405] +
                 0.0020280330861290126 * v[427] -
                 0.0006678100902549639 * v[469] -
                 8.895004679857398e-6 * v[491]);
    x[2 * 28] =
      mk_sample (0.0002609184066830402 * v[20] + 0.0017434010558554753 * v[44] +
                 0.00335074197528923 * v[84] + 0.019131532006261515 * v[108] +
                 0.05087908750324039 * v[148] + 0.07792084947107533 * v[172] +
                 0.4135144655813375 * v[212] + 0.6818808379899038 * v[236] -
                 0.6155065051657198 * v[276] + 0.03464780080203036 * v[300] -
                 0.09048375437030715 * v[340] + 0.016718298292953078 * v[364] -
                 0.009763841144830749 * v[404] +
                 0.0020829053453968282 * v[428] -
                 0.000563034124956373 * v[468] - 9.175921629741856e-6 * v[492]);
    x[2 * 29] =
      mk_sample (0.00026895700362872434 * v[19] +
                 0.0018617373983575926 * v[45] + 0.0037653930147042687 * v[83] +
                 0.01982370169095419 * v[109] + 0.0520367067180179 * v[147] +
                 0.07686314941070457 * v[173] + 0.40915989856270835 * v[211] +
                 0.7088849290922274 * v[237] - 0.5507975895314607 * v[275] +
                 0.04332338422655655 * v[301] - 0.0798801226409765 * v[339] +
                 0.0179809618734234 * v[365] - 0.008260811222858336 * v[403] +
                 0.0021371985853214895 * v[429] -
                 0.0004866834622998666 * v[467] - 9.49879041340121e-6 * v[493]);
    x[2 * 30] =
      mk_sample (0.0002886308928087058 * v[18] + 0.0019936826123059135 * v[46] +
                 0.004173121100249679 * v[82] + 0.02052901151720049 * v[110] +
                 0.05321631209338495 * v[146] + 0.07540463876838151 * v[174] +
                 0.40638020126565166 * v[210] + 0.7388075471885076 * v[238] -
                 0.49519431690415505 * v[274] + 0.052191064589741536 * v[302] -
                 0.07070253046441433 * v[338] + 0.019265687721575508 * v[366] -
                 0.0069872713199412816 * v[402] +
                 0.002191077096864582 * v[430] -
                 0.0004209202818911278 * v[466] -
                 9.869853592290089e-6 * v[494]);
    x[2 * 31] =
      mk_sample (0.0002953791302584709 * v[17] + 0.0021417256087434577 * v[47] +
                 0.004555649177065818 * v[81] + 0.02124221215327783 * v[111] +
                 0.05439513647513115 * v[145] + 0.07345707245390273 * v[175] +
                 0.4048961286830606 * v[209] + 0.7721744861726333 * v[239] -
                 0.4468853573225977 * v[273] + 0.06135838173971364 * v[303] -
                 0.06267710257781978 * v[337] + 0.02059351619540377 * v[367] -
                 0.005896215025383755 * v[401] +
                 0.0022446932639493943 * v[431] -
                 0.0003521820359820493 * v[465] -
                 0.000010296900482261932 * v[495]);
}
}

void
dct32 (const double *y, double *v)
{
  {
    double f0, f1, f2, f3, f4, f5, f6, f7, f8, f9, f10, f11, f12, f13, f14, f15, f16, f17, f18, f19,    /* 411 */
     
      f20, f21, f22, f23, f24, f25, f26, f27, f28, f29, f30, f31, f32, f33, f34,
      f35, f36, f37, f38, f39, f40, f41, f42, f43, f44, f45, f46, f47, f48, f49,
      f50, f51, f52, f53, f54, f55, f56, f57, f58, f59, f60, f61, f62, f63, f64,
      f65, f66, f67, f68, f69, f70, f71, f72, f73, f74, f75, f76, f77, f78, f79,
      f80, f81, f82, f83, f84, f85, f86, f87, f88, f89, f90, f91, f92, f93, f95,
      f96, f97, f98, f99, f100, f101, f102, f103, f104, f105, f106, f107, f108,
      f109, f110, f111, f112, f113, f114, f115, f116, f117, f118, f119, f120,
      f121, f122, f123, f124, f125, f126, f127, f128, f129, f130, f131, f132,
      f133, f134, f135, f151, f152, f153, f154, f155, f156, f157, f158, f159,
      f160, f161, f162, f163, f164, f165, f166, f167, f168, f169, f170, f171,
      f172, f173, f174, f175, f176, f177, f178, f179, f180, f181, f182, f183,
      f184, f185, f186, f187, f188, f189, f190, f191, f192, f193, f194, f195,
      f196, f197, f198, f199, f200, f201, f202, f203, f204, f205, f206, f207,
      f208, f209, f210, f211, f212, f213, f214, f215, f216, f217, f218, f219,
      f220, f221, f222, f223, f224, f225, f226, f227, f228, f229, f230, f231,
      f232, f233, f234, f235, f236, f237, f238, f239, f240, f241, f242, f243,
      f244, f245, f246, f247;
    f0 = (y[31]) + (y[30]);
    f1 = (y[30]) + (y[29]);
    f2 = (y[29]) + (y[28]);
    f3 = (y[28]) + (y[27]);
    f4 = (y[27]) + (y[26]);
    f5 = (y[26]) + (y[25]);
    f6 = (y[25]) + (y[24]);
    f7 = (y[24]) + (y[23]);
    f8 = (y[23]) + (y[22]);
    f9 = (y[22]) + (y[21]);
    f10 = (y[21]) + (y[20]);
    f11 = (y[20]) + (y[19]);
    f12 = (y[19]) + (y[18]);
    f13 = (y[18]) + (y[17]);
    f14 = (y[17]) + (y[16]);
    f15 = (y[16]) + (y[15]);
    f16 = (y[15]) + (y[14]);
    f17 = (y[14]) + (y[13]);
    f18 = (y[13]) + (y[12]);
    f19 = (y[12]) + (y[11]);
    f20 = (y[11]) + (y[10]);
    f21 = (y[10]) + (y[9]);
    f22 = (y[9]) + (y[8]);
    f23 = (y[8]) + (y[7]);
    f24 = (y[7]) + (y[6]);
    f25 = (y[6]) + (y[5]);
    f26 = (y[5]) + (y[4]);
    f27 = (y[4]) + (y[3]);
    f28 = (y[3]) + (y[2]);
    f29 = (y[2]) + (y[1]);
    f30 = (y[1]) + (y[0]);
    f46 = (y[0]) - (y[31]);
    f47 = (y[0]) + (y[31]);
    f31 = f23 - f7;
    f32 = f24 - f6;
    f33 = f22 - f8;
    f34 = f25 - f5;
    f35 = f21 - f9;
    f36 = f26 - f4;
    f37 = f20 - f10;
    f38 = f27 - f3;
    f39 = f19 - f11;
    f40 = f28 - f2;
    f41 = f18 - f12;
    f42 = f29 - f1;
    f43 = f17 - f13;
    f44 = f30 - f0;
    f45 = f16 - f14;
    f48 = f30 + f0;
    f49 = f28 + f2;
    f50 = f26 + f4;
    f51 = f24 + f6;
    f52 = f22 + f8;
    f53 = f20 + f10;
    f54 = f18 + f12;
    f55 = f16 + f14;
    f56 = f29 + f1;
    f57 = f25 + f5;
    f58 = f21 + f9;
    f59 = f17 + f13;
    f60 = f27 + f3;
    f61 = f19 + f11;
    f62 = f47 - f15;
    f63 = f47 + f15;
    f64 = f23 + f7;
    f65 = f63 + f64;
    f66 = f63 - f64;
    f67 = f60 - f61;
    f68 = f60 + f61;
    f69 = 0.7071067811865476 * f67;
    f70 = f65 + f68;
    f75 = f56 + f59;
    f76 = f57 - f58;
    f77 = f57 + f58;
    f79 = f75 + f77;
    v[16] = f70 - f79;
    f78 = f75 - f77;
    f71 = f62 - f69;
    f72 = f62 + f69;
    f73 = f65 - f68;
    f74 = f56 - f59;
    f80 = 0.7071067811865476 * f78;
    f81 = f74 + f76;
    f82 = 1.3065629648763766 * f74;
    f83 = (-0.9238795325112866) * f81;
    f84 = (-0.5411961001461967) * f76;
    f85 = f82 + f83;
    f86 = f84 - f83;
    f87 = f70 + f79;
    f88 = f71 - f85;
    f90 = f66 - f80;
    f91 = f66 + f80;
    f89 = f72 + f86;
    f92 = f72 - f86;
    f93 = f71 + f85;
    f95 = f48 - f55;
    f96 = f48 + f55;
    f97 = f49 - f54;
    f98 = f49 + f54;
    f99 = f50 - f53;
    f100 = f50 + f53;
    f101 = f51 - f52;
    f102 = f51 + f52;
    f103 = f96 - f102;
    f104 = f96 + f102;
    f105 = f98 - f100;
    f106 = f98 + f100;
    f107 = f104 - f106;
    f108 = f104 + f106;
    v[0] = f87 + f108;
    f109 = 0.7071067811865476 * f107;
    v[24] = f73 - f109;
    v[8] = f73 + f109;
    f110 = f103 + f105;
    f111 = 1.3065629648763766 * f103;
    f112 = (-0.9238795325112866) * f110;
    f113 = (-0.5411961001461967) * f105;
    f114 = f111 + f112;
    v[20] = f90 - f114;
    v[12] = f90 + f114;
    f115 = f113 - f112;
    v[4] = f91 + f115;
    v[28] = f91 - f115;
    f116 = f97 - f99;
    f117 = f97 + f99;
    f118 = 0.7071067811865476 * f117;
    f119 = 0.7071067811865476 * f116;
    f120 = f95 - f118;
    f121 = f95 + f118;
    f122 = f101 - f119;
    f123 = f101 + f119;
    f124 = f123 + f121;
    f126 = 0.9807852804032304 * f124;
    f125 = (-0.7856949583871021) * f123;
    f127 = 1.1758756024193588 * f121;
    f128 = f125 + f126;
    v[2] = f89 + f128;
    v[30] = f89 - f128;
    f129 = f127 - f126;
    v[18] = f92 - f129;
    v[14] = f92 + f129;
    f130 = f122 + f120;
    f132 = 0.5555702330196022 * f130;
    f131 = 0.2758993792829431 * f122;
    f133 = 1.3870398453221475 * f120;
    f134 = f131 + f132;
    v[22] = f88 - f134;
    v[10] = f88 + f134;
    f135 = f133 - f132;
    v[6] = f93 + f135;
    v[26] = f93 - f135;
    f151 = 0.7071067811865476 * f31;
    f152 = f46 - f151;
    f153 = f46 + f151;
    f154 = f38 + f39;
    f155 = 1.3065629648763766 * f38;
    f156 = (-0.9238795325112866) * f154;
    f157 = (-0.5411961001461967) * f39;
    f158 = f155 + f156;
    f159 = f157 - f156;
    f160 = f153 - f159;
    f161 = f153 + f159;
    f162 = f152 - f158;
    f163 = f152 + f158;
    f164 = f34 - f35;
    f165 = f34 + f35;
    f166 = 0.7071067811865476 * f165;
    f167 = 0.7071067811865476 * f164;
    f168 = f42 - f166;
    f169 = f42 + f166;
    f170 = f43 - f167;
    f171 = f43 + f167;
    f172 = f171 + f169;
    f173 = (-0.7856949583871021) * f171;
    f174 = 0.9807852804032304 * f172;
    f175 = 1.1758756024193588 * f169;
    f176 = f173 + f174;
    f177 = f175 - f174;
    f178 = f170 + f168;
    f179 = 0.2758993792829431 * f170;
    f180 = 0.5555702330196022 * f178;
    f181 = 1.3870398453221475 * f168;
    f182 = f179 + f180;
    f183 = f181 - f180;
    f184 = f161 - f176;
    f185 = f161 + f176;
    f186 = f163 - f183;
    f187 = f163 + f183;
    f188 = f162 - f182;
    f189 = f162 + f182;
    f190 = f160 - f177;
    f191 = f160 + f177;
    f192 = f44 - f40;
    f193 = f36 - f40;
    f194 = f36 - f32;
    f195 = f33 - f32;
    f196 = f33 - f37;
    f197 = f41 - f37;
    f198 = f41 - f45;
    f199 = f196 - f194;
    f200 = f196 + f194;
    f201 = 0.7071067811865476 * f200;
    f202 = f198 - f201;
    f203 = f198 + f201;
    f204 = 0.7071067811865476 * f199;
    f205 = f204 - f192;
    f206 = f204 + f192;
    f207 = f206 + f203;
    f208 = (-0.7856949583871021) * f206;
    f209 = 0.9807852804032304 * f207;
    f210 = 1.1758756024193588 * f203;
    f211 = f208 + f209;
    f212 = f210 - f209;
    f213 = f205 + f202;
    f214 = (-0.2758993792829429) * f205;
    f215 = 0.8314696123025452 * f213;
    f216 = 1.3870398453221475 * f202;
    f217 = f214 + f215;
    f218 = f216 - f215;
    f219 = f193 + f197;
    f222 = 0.5411961001461961 * f197;
    f221 = (-0.3826834323650904) * f219;
    f220 = 1.3065629648763770 * f193;
    f223 = f220 + f221;
    f224 = f222 - f221;
    f225 = 0.7071067811865476 * f195;
    f226 = f225 + f45;
    f227 = f225 - f45;
    f228 = f224 - f227;
    f229 = f224 + f227;
    f232 = f211 + f229;
    f233 = f211 - f229;
    f244 = 0.5024192861881557 * f232;
    v[17] = f184 - f244;
    v[15] = f184 + f244;
    f240 = 5.1011486186891641 * f233;
    v[31] = f185 - f240;
    v[1] = f185 + f240;
    f230 = f223 - f226;
    f231 = f223 + f226;
    f234 = f231 - f217;
    f235 = f217 + f231;
    f241 = 1.7224470982383342 * f235;
    v[29] = f187 + f241;
    v[3] = f187 - f241;
    f236 = f218 + f230;
    f237 = f218 - f230;
    f242 = 1.0606776859903475 * f237;
    v[27] = f189 - f242;
    v[5] = f189 + f242;
    f238 = f228 - f212;
    f239 = f212 + f228;
    f243 = 0.7881546234512502 * f239;
    v[25] = f191 + f243;
    v[7] = f191 - f243;
    f247 = 0.6468217833599901 * f238;
    v[23] = f190 - f247;
    v[9] = f190 + f247;
    f245 = 0.5224986149396889 * f234;
    v[19] = f186 - f245;
    v[13] = f186 + f245;
    f246 = 0.5669440348163577 * f236;
    v[21] = f188 - f246;
    v[11] = f188 + f246;
  }
}

#define DK(name, value)  const double name =  ((double)(value))      /* 413 */
void
dct18 (const double *I, double *O)
{
  double I17, I16, I15, I14, I13, I12, I11, I10, I9, I8, I7, I6, I5, I4, I3, I2,
    I1, I0;
  DK (KP173648177, +0.173648177666930348851716626769314796000375677);
  DK (KP984807753, +0.984807753012208059366743024589523013670643252);
  DK (KP642787609, +0.642787609686539326322643409907263432907559884);
  DK (KP766044443, +0.766044443118978035202392650555416673935832457);
  DK (KP939692620, +0.939692620785908384054109277324731469936208135);
  DK (KP342020143, +0.342020143325668733044099614682259580763083368);
  DK (KP1_732050807, +1.732050807568877293527446341505872366942805254);
  DK (KP1_224744871, +1.224744871391589049098642037352945695982973740);
  DK (KP707106781, +0.707106781186547524400844362104849039284835938);
  DK (KP1_414213562, +1.414213562373095048801688724209698078569671875);
  I17 = I[17] + I[16];
  I16 = I[16] + I[15];
  I15 = I[15] + I[14];
  I14 = I[14] + I[13];
  I13 = I[13] + I[12];
  I12 = I[12] + I[11];
  I11 = I[11] + I[10];
  I10 = I[10] + I[9];
  I9 = I[9] + I[8];
  I8 = I[8] + I[7];
  I7 = I[7] + I[6];
  I6 = I[6] + I[5];
  I5 = I[5] + I[4];
  I4 = I[4] + I[3];
  I3 = I[3] + I[2];
  I2 = I[2] + I[1];
  I1 = I[1] + I[0];
  I0 = I[0];
  {
    double f0, f1, f2, f3, f4, f5, f6, f7, f8, f9, f10, f11, f12, f13, f14, f15,
      f16, f17, f18, f19, f20, f21, f22, f23;
    f22 = I0 + I0;
    f23 = KP1_414213562 * I9;
    f0 = f22 - f23;
    f1 = f22 + f23;
    {
      double g0, g1;

      g0 = KP707106781 * (I17 + I1);
      g1 = KP707106781 * (I17 - I1);
      f2 = I8 - g0;
      f3 = g1 + I10;
      f4 = I8 + g0;
      f5 = g1 - I10;
    }
    {
      double g1, g2, g3;

      g1 = KP1_732050807 * I6;
      g2 = KP707106781 * (I15 - I3);
      g3 = KP1_224744871 * (I15 + I3);
      f6 = g2 + I12;
      f7 = g3 - g1;
      f8 = g2 - I12;
      f9 = g3 + g1;
    }
    {
      double g4, g5, g6, g7;

      g4 = KP707106781 * (I7 - I11);
      g5 = KP707106781 * (I7 + I11);
      g6 = KP707106781 * (I5 + I13);
      g7 = KP707106781 * (I13 - I5);
      f10 = g6 - I4;
      f11 = I16 - g4;
      f12 = f10 + f11;
      f13 = g5 - I2;
      f14 = g7 + I14;
      f15 = f13 + f14;
      f16 = I4 + g6;
      f17 = I16 + g4;
      f18 = f16 - f17;
      f19 = g5 + I2;
      f20 = g7 - I14;
      f21 = f19 + f20;
    }
    {
      double g0, g1, g2, g3;

      g0 = KP1_732050807 * (f15 + f3);
      g1 = f0 - f6 - f6;
      g2 = f2 + f12;
      g3 = g1 - g2;
      O[13] = g1 + g2 + g2;
      O[10] = g3 + g0;
      O[1] = g3 - g0;
    }
    {
      double g0, g1, g2, g3;

      g0 = KP1_732050807 * (f21 + f5);
      g1 = f1 + f8 + f8;
      g2 = f4 - f18;
      g3 = g1 - g2;
      O[4] = g1 + g2 + g2;
      O[16] = g3 + g0;
      O[7] = g3 - g0;
    }
    {
      double g0, g1, g2, g3, g4, g5, g6;

      g6 = f1 - f8;
      g0 = g6 + f9;
      g1 = g6 - f9;
      {
        double h0, h1, h2, h3;

        h0 = f18 + f4 + f4;
        h1 = KP1_732050807 * (f19 - f20);
        g2 = h0 + h1;
        g3 = h0 - h1;
        h2 = f5 + f5 - f21;
        h3 = KP1_732050807 * (f16 + f17);
        g4 = h2 - h3;
        g5 = h3 + h2;
      }
      O[11] = g0 - KP342020143 * g4 - KP939692620 * g2;
      O[0] = g0 + KP766044443 * g2 - KP642787609 * g4;
      O[8] = g1 + KP766044443 * g3 + KP642787609 * g5;
      O[15] = g1 + KP342020143 * g5 - KP939692620 * g3;
      O[3] = g1 + KP173648177 * g3 - KP984807753 * g5;
      O[12] = g0 + KP173648177 * g2 + KP984807753 * g4;
    }
    {
      double g0, g1, g2, g3, g4, g5, g6;

      g6 = f0 + f6;
      g0 = g6 + f7;
      g1 = g6 - f7;
      {
        double h0, h1, h2, h3;

        h0 = f2 + f2 - f12;
        h1 = KP1_732050807 * (f14 - f13);
        g2 = h0 - h1;
        g3 = h0 + h1;
        h2 = f15 - f3 - f3;
        h3 = KP1_732050807 * (f10 - f11);
        g4 = h2 - h3;
        g5 = h3 + h2;
      }
      O[2] = g0 + KP342020143 * g4 - KP939692620 * g2;
      O[14] = g0 + KP173648177 * g2 - KP984807753 * g4;
      O[5] = g1 + KP173648177 * g3 + KP984807753 * g5;
      O[17] = g1 + KP766044443 * g3 - KP642787609 * g5;
      O[6] = g1 - KP342020143 * g5 - KP939692620 * g3;
      O[9] = g0 + KP766044443 * g2 + KP642787609 * g4;
    }
  }
}

void                                                                 /* 414 */
dct6 (const double *z, double *t)
{
  double f0,
    f1,
    f2,
    f3,
    f4,
    f5, f6, f7, f8, f9, f10, f11, f12, f13, f14, f15, f16, f17, f18, f19, f20;
  f0 = z[0 * 3] + z[1 * 3];
  f1 = z[1 * 3] + z[2 * 3];
  f2 = z[2 * 3] + z[3 * 3];
  f3 = z[3 * 3] + z[4 * 3];
  f4 = z[4 * 3] + z[5 * 3];
  f5 = f0 + f4;
  f6 = f4 - f0;
  f7 = f2 + f2;
  f8 = f7 - f6;
  f9 = f2 + f6;
  f10 = z[0 * 3] - f3;
  f11 = 0.5000000000000000 * f3;
  f12 = z[0 * 3] + f11;
  f13 = (-0.8660254037844386) * f1;
  f14 = f13 + f12;
  f15 = f12 - f13;
  f16 = 0.3535533905932737 * f8;
  f17 = (-0.7071067811865475) * f9;
  f18 = 0.6123724356957945 * f5;
  f19 = f16 + f18;
  f20 = f18 - f16;
  t[0] = f19 + f15;
  t[1] = f17 + f10;
  t[2] = f20 + f14;
  t[3] = f14 - f20;
  t[4] = f10 - f17;
  t[5] = f15 - f19;
}
